﻿Function Get-IPConfig(){

	<#
     .SYNOPSIS
		Permet de récupérer la configuration IP de la machine locale ou d'une machine distante, pour chaque carte réseau active.
     .DESCRIPTION
        La cmdlet Get-IPConfig permet de récupérer la configuration IP d'une machine pour chaque carte réseau active.
		Les informations sont récupérées via WMI et les classes Win32_NetworkAdapterConfiguration, Win32_NetworkAdapter, 
		Win32_ComputerSystem et StdRegProv.
		
		Pour chaque machine joignable et chaque interface réseau active, un objet PowerShell personnalisé (PSObject) est renvoyé.
		
		Pour effectuer un simple "ipconfig /all" sur une machine distante, il est également possible d'utiliser les commandes suivantes :
		
			PS C:\> $Credential = Get-Credential
			PS C:\> Invoke-Command -ComputerName CATOSV01 -ScriptBlock {ipconfig /all} -Credential $credential
		
		Mais dans ce cas, l'activation de WinRM sur la machine distante est obligatoire. 
		De plus, cette commande ne revoie pas d'objet personnalisé mais un tableau de 'system.string' correspondant à la sortie de la commande ipconfig
		donc plus difficile à exploiter.
							
     .PARAMETER  ComputerName
		Spécifie l'ordinateur sur lequel vous voulez exécuter l'opération de gestion. La valeur peut être un nom de domaine
		complet, un nom NetBIOS ou une adresse IP. Utilisez le nom de l'ordinateur local, localhost ou un point (.) pour 
		spécifier l'ordinateur local. L'ordinateur local est la valeur par défaut. Lorsque l'ordinateur distant se trouve 
		dans un domaine différent de l'utilisateur, vous devez utiliser un nom de domaine complet. Ce paramètre peut également 
		être redirigé vers l'applet de commande.

		Ce paramètre ne s'appuie pas sur la communication à distance Windows PowerShell, laquelle utilise le service Gestion
		des services Web). Vous pouvez utiliser le paramètre ComputerName de Get-IPConfig même si votre ordinateur n'est
		pas configuré pour exécuter des commandes distantes du service Gestion des services Web.
	 .PARAMETER Credential
	 	Spécifie un compte d'utilisateur qui a l'autorisation d'exécuter cette action. La valeur par défaut est l'utilisateur actuel. 
		Tapez un nom d'utilisateur, tel que « User01 », « Domain01\User01 » ou « User@Contoso.com ». Vous pouvez également entrer un objet 
		PSCredential, tel qu'un objet qui est retourné par l'applet de commande Get-Credential. Lorsque vous tapez un nom d'utilisateur, 
		vous êtes invité à entrer un mot de passe.
		Si le pramétre '-Credential' n'est pas spécifié, le compte courant est utilisé pour réaliser les opérations via WMI.
	.EXAMPLE
		  PS C:\> Get-IPConfig
          PS C:\> Get-IPConfig localhost
		  PS C:\> Get-IPConfig .
		  
		Ces commandes récupèrent la configuration IP de la machine locale, pour toutes les cartes réseau actives.
	 .EXAMPLE
          PS C:\> Get-IPConfig CATOSV01
		  PS C:\> Get-IPConfig 192.168.1.23
		  
		Ces commandes récupèrent la configuration IP de la machine distante, soit par son nom, soit par son adresse IP, 
		pour toutes les cartes réseau actives.
	 .EXAMPLE
          PS C:\> $Credential = Get-Credential
		  PS C:\> Get-IPConfig CATOSV01 -Credential $Credential
		  
		La première commande récupère un Credential.
		La seconde commande récupère la configuration IP de la machine distante	pour toutes les cartes réseau actives, en utilisant le Credential
		spécifié.
	 .EXAMPLE
          PS C:\> $Credential = Get-Credential
		  PS C:\> "CATOSV01", "CATODC01", CATOSV03" | Get-IPConfig -Credential $Credential
		  
		La première commande récupère un Credential.
		La seconde commande récupère la configuration IP des machines distantes spécifiées par le pipeline, pour toutes les cartes réseau actives et
		en utilisant le Credential spécifié.
	 .EXAMPLE
          PS C:\> $Credential = Get-Credential
		  PS C:\> (1..254) | Get-IPConfig -ComputerName {"192.168.1.$_"} -Credential $Credential
		  
		La première commande récupère un Credential.
		La seconde commande récupère la configuration IP de toutes les machines dont l'adresse IP est comprise entre 192.168.1.1 et 192.168.1.254.
	 .OUTPUTS
	 	Selected.System.Management.Automation.PSCustomObject
     .LINK
	 	http://msdn.microsoft.com/en-us/library/windows/desktop/aa394217(v=vs.85).aspx
		http://msdn.microsoft.com/en-us/library/windows/desktop/aa394216(v=vs.85).aspx
		http://msdn.microsoft.com/en-us/library/windows/desktop/aa394102(v=vs.85).aspx
		http://msdn.microsoft.com/en-us/library/windows/desktop/aa393664(v=vs.85).aspx
		www.via-powershell.fr
		www.powershell-scripting.com
	#>

	[CmdletBinding()]
	param(
		[Parameter(Position=0,Mandatory=$False,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
		[Alias("Name")]
		[String[]]$ComputerName = ".",
		[parameter(Mandatory=$false)]
		[System.Management.Automation.PSCredential]$Credential = [System.Management.Automation.PSCredential]::Empty
	)

	Begin{
		Function Get-NetworkAdapterInfos($ComputerName, $ComputerSystemInfos, $Nic, $Configuration, $SearchList){
			Write-Debug "Appel de la fonction 'Get-NetworkAdapterInfos' ..."
			$Index = $Nic.Index
			$AdapterType = $Nic.AdapterType
			$Connecteur = $Nic.NetConnectionID
			
			Write-Debug "Création d'un objet PowerShell personnalisé ..."
			$NewConfiguration = New-Object PSObject -Property @{
														HostName = $ComputerName
														SearchList = $SearchList
														Index = $Index
														AdaptaterType = $Nic.AdapterType
														ConnexionType = $Connecteur
														DNSDomain = $ComputerSystemInfos.Domain
														Description = $Nic.Description
														MACAddress = $Nic.MACAddress
														DHCPEnabled = $Configuration.DHCPEnabled
														IPAddress = $Configuration.IPAddress
														IPSubnet = $Configuration.IPSubnet
														DefaultIPGateway = $Configuration.DefaultIPGateway
														DNSServers = $Configuration.DNSServerSearchOrder
														WINSPrimaryServer = $Configuration.WINSPrimaryServer
														WINSSecondaryServer = $Configuration.WINSSecondaryServer
													}
			if($Configuration.DHCPEnabled){
				Write-Debug "La carte réseau est configurée en DHCP ..."
				try{
					$DHCPLeaseObtained = $Configuration.ConvertToDateTime($Configuration.DHCPLeaseObtained)
				} 
				catch{
					$DHCPLeaseObtained = ""
				}
				try{
					$DHCPLeaseExpires = $Configuration.ConvertToDateTime($Configuration.DHCPLeaseExpires)
				}
				catch{
					$DHCPLeaseExpires = ""
				}
				
				Add-Member -InputObject $NewConfiguration -MemberType NoteProperty -Name DHCPServer -Value ($Configuration.DHCPServer)
				Add-Member -InputObject $NewConfiguration -MemberType NoteProperty -Name DHCPLeaseObtained -Value $DHCPLeaseObtained
				Add-Member -InputObject $NewConfiguration -MemberType NoteProperty -Name DHCPLeaseExpires -Value $DHCPLeaseExpires
				return $NewConfiguration | 
				Select-Object Description,Index,AdaptaterType,ConnexionType,HostName,MACAddress,IPAddress,IPSubnet,DefaultIPGateway `
								,DNSDomain,SearchList,DNSServers,WINSPrimaryServer,WINSSecondaryServer,DHCPEnabled, DHCPServer, DHCPLeaseObtained, DHCPLeaseExpires
			}
			else{
				Write-Debug "La carte réseau n'est pas configurée en DHCP ..."
				return $NewConfiguration | 
				Select-Object Description,Index,AdaptaterType,ConnexionType,HostName,MACAddress,IPAddress,IPSubnet,DefaultIPGateway `
								,DNSDomain,SearchList,DNSServers,WINSPrimaryServer,WINSSecondaryServer,DHCPEnabled
			}
		
		}
	
		Function GetIPConfig([String]$ComputerName, [System.Management.Automation.PSCredential]$Credential){
			Write-Debug "Récupération des informations de carte réseau via WMI ..."
			try{
				$NetworkAdapterConfiguration = Get-WmiObject -Class Win32_NetworkAdapterConfiguration -ComputerName $ComputerName -Credential $Credential -ErrorAction Stop
				$NetworkAdapter = Get-WmiObject -Class Win32_NetworkAdapter -ComputerName $ComputerName -Credential $Credential -ErrorAction Stop
				$ComputerSystemInfos = Get-WmiObject -Class Win32_ComputerSystem -ComputerName $ComputerName -Credential $Credential -ErrorAction Stop
			}
			catch{
				Write-Error "Une erreur s'est produite lors de la connexion en WMI sur '$ComputerName' : $($_.Exception.Message)"
				return $null
			}
			
			Write-Debug "Récupération de la liste de recherche de suffixe DNS via WMI / StdRegProv ..."
			try{
				$objreg = Get-WmiObject -list -namespace root\default -computerName $ComputerName -credential $Credential -ErrorAction Stop | where-object { $_.name -eq "StdRegProv" }
			}
			catch{
				Write-Error "Une erreur s'est produite lors de la connexion au registre distant de '$ComputerName' via WMI : $($_.Exception.Message)"
				return $null
			}
			
			$hklm = 2147483650
			$key = "SOFTWARE\Policies\Microsoft\Windows NT\DNSClient"
			$value = "SearchList"
			$SearchList = $null
			
			Write-Debug "Lecture de la valeur du registre 'HKLM\$key\$value' ..."
			try{
				$SearchList = ($objreg.GetStringValue($hklm,$key,$value).svalue).split(",")
			}
			catch{
				$SearchList = $null
			}
			
			if($SearchList -eq $null){
				$key = "SYSTEM\CurrentControlSet\Services\Tcpip\Parameters"
				Write-Debug "Pas de stratégie pour la liste de recherche de suffixes DNS : Lecture de la valeur du registre 'HKLM\$key\$value' ..."
				try{
					$SearchList = ($objreg.GetStringValue($hklm,$key,$value).svalue).split(",")
				}
				catch{
					$SearchList = ""
				}
			}
			
			$Count = $NetworkAdapter.Count
			
			Write-Debug "'$Count' interfaces réseau ont été identifiées sur '$ComputerName'"
			
			For ($i=0 ; $i -lt $Count ; $i++) {
				$Nic = $NetworkAdapter[$i]
				$Configuration = $NetworkAdapterConfiguration[$i]
				
				If ($Configuration.IPEnabled) {
					Write-Debug "L'interface '$($Nic.Description)' est active"
					Write-Debug "Récupération des informations IP pour cette interface ..."
					Get-NetworkAdapterInfos $ComputerName $ComputerSystemInfos $Nic $Configuration $SearchList
				}
				else{
					Write-Debug "L'interface '$($Nic.Description)' n'est pas active"
				}
			}
		}
	}
	
	Process{
		foreach($Computer in $ComputerName){
			if($Computer -eq "."){
				$Host = $env:COMPUTERNAME
			}
			else{
				$Host = $Computer
			}
			if(Test-Connection -ComputerName $Host -Count 1 -Quiet){
				GetIPConfig -ComputerName $Computer -Credential $Credential
			}
			else{
				Write-Warning "'$Computer' est injoignable"
			}
		}
	}

}

Function Resolve-Computer{

	<#
     .SYNOPSIS
		Permet de résoudre une machine à partir de son nom ou de son adresse IP.
     .DESCRIPTION
        La cmdlet Resolve-Computer permet de résoudre une machine à partir de son nom ou de son adresse IP.
		Si la machine répond, elle renvoie un objet PowerShell personnalisé dont les propriétés sont le nom (DNS HostName) et l'adresse IP.
		Si la machine ne répond pas, $null est renvoyé.
		Cette Cmdlet n'est pas exportée et reste donc privée au module.
     .PARAMETER  ComputerName
		Spécifie l'ordinateur sur lequel vous voulez exécuter l'opération de gestion. La valeur peut être un nom de domaine
		complet, un nom NetBIOS ou une adresse IP. Utilisez le nom de l'ordinateur local, localhost ou un point (.) pour 
		spécifier l'ordinateur local. L'ordinateur local est la valeur par défaut. Lorsque l'ordinateur distant se trouve 
		dans un domaine différent de l'utilisateur, vous devez utiliser un nom de domaine complet. Ce paramètre peut également 
		être redirigé vers l'applet de commande.

		Ce paramètre ne s'appuie pas sur la communication à distance Windows PowerShell, laquelle utilise le service Gestion
		des services Web). Vous pouvez utiliser le paramètre ComputerName de Test-TCPPorts même si votre ordinateur n'est
		pas configuré pour exécuter des commandes distantes du service Gestion des services Web.
     .EXAMPLE
          PS C:\> Resolve-Computer 10.34.145.12
		  PS C:\> Resolve-Computer MonNomDeServeur
		  
		Si les machines répondent, ces commandes renvoient un objet PowerShell personnalisé dont les propriétés sont le nom (DNS HostName)
		et l'adresse IP.
     .OUTPUTS
	 	System.Management.Automation.PSCustomObject
     .LINK
		http://msdn.microsoft.com/en-us/library/system.net.dns.resolve(v=vs.80).aspx
		www.via-powershell.fr
		www.powershell-scripting.com
	#>

	[CmdletBinding()]
	param(
		[Parameter(Position=0,Mandatory=$True,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
        [Alias("Name")]
		[String]$ComputerName
	)
	
	if($ComputerName -eq "localhost" -or $ComputerName -eq "127.0.0.1" -or $ComputerName -eq "."){
		$ComputerName = $env:COMPUTERNAME
	}
	
	Write-Debug "Contrôle que la machine '$ComputerName' répond ..."
	$Alive = Test-Connection -ComputerName $ComputerName -Count 1 -Quiet

	if($Alive){

		Write-Debug "Résolution du nom ou de l'adresse IP : '$ComputerName'"
		try{
			$Resolve = [System.Net.Dns]::Resolve($ComputerName)
			$IpAddress = $Resolve.AddressList[0].IPAddressToString
			$HostName = $Resolve.HostName
		}
		catch{
			Write-Error "Une erreur s'est produite lors de la résolution de '$ComputerName' : $($_.Exception.Message)"
			return $null
		}
		
		Write-Debug "L'adresse IP de '$ComputerName' est : $IpAddress"
		Write-Debug "Le nom DNS de '$ComputerName' est : $HostName"
			
		$objComputer = New-Object PSObject -Property @{IpAddress = $IpAddress;HostName = $HostName}
		
		return $objComputer
	}
	else{
		Write-Warning "'$ComputerName' est injoignable"
		return $null
	}

}

Function Test-TCPPorts{

	<#
     .SYNOPSIS
		Permet de récupérer des informations sur les ports TCP ouverts / fermés, à partir d'une adresse IP ou d'un nom d'équipement.
     .DESCRIPTION
        La cmdlet Test-TCPPorts permet de déterminer les ports TCP ouverts et fermés, selon une liste prédéfinie par défaut :
		80 (HTTP),21 (FTP),22 (SSH),23 (Telnet),389 (LDAP),443 (HTTPS),135 (EPMAP / DCOM),139 (NetBios),445 (Windows Shares),636 (LDAP over SSL),1433 (MSSQL) et 3389 (RDP).
		
		S'agissant d'un module, il est possible d'appeler directement les sous fonctions appelées dans ce module, en utilisant leurs cmdlets respectives : 
		Test-Ports et Get-NetBiosInfos.
		
		Si l'on souhaite tester des ports particuliers, il est donc possible d'utiliser dirrectement la cmdlet Test-TCPPorts et son paramètre -portlist, l'argument étant une liste de ports telle que @(21,80,445).
				
     .PARAMETER  ComputerName
		Spécifie l'ordinateur sur lequel vous voulez exécuter l'opération de gestion. La valeur peut être un nom de domaine
		complet, un nom NetBIOS ou une adresse IP. Utilisez le nom de l'ordinateur local, localhost ou un point (.) pour 
		spécifier l'ordinateur local. L'ordinateur local est la valeur par défaut. Lorsque l'ordinateur distant se trouve 
		dans un domaine différent de l'utilisateur, vous devez utiliser un nom de domaine complet. Ce paramètre peut également 
		être redirigé vers l'applet de commande.

		Ce paramètre ne s'appuie pas sur la communication à distance Windows PowerShell, laquelle utilise le service Gestion
		des services Web). Vous pouvez utiliser le paramètre ComputerName de Test-TCPPorts même si votre ordinateur n'est
		pas configuré pour exécuter des commandes distantes du service Gestion des services Web.
	 .PARAMETER TCPPortList
	 	Spécifie un port ou une liste de ports.
		La liste de ports prédéfinie par défaut est : 80 (HTTP),21 (FTP),22 (SSH),23 (Telnet),389 (LDAP),443 (HTTPS),135 (EPMAP / DCOM),139 (NetBios),445 (Windows Shares),636 (LDAP over SSL),1433 (MSSQL) et 3389 (RDP).
	 .PARAMETER TimeOut
	 	Permet de fixer le Timeout de la requête TCP. La valeur indiquée est en ms. Par défaut, le Timeout est fixé à 800 ms.
		Si le délai spécifié est trop court, les ports peuvent alors apparaîtrent fermés alors que ce n'est pas le cas.
     .EXAMPLE
          PS C:\> Test-TCPPorts
		  PS C:\> Test-TCPPorts .
		  PS C:\> Test-TCPPorts localhost
		  
		Ces commandes récupèrent la liste des ports ouverts et des ports fermés sur la machine locale.
		Les ports testés sont : 80 (HTTP),21 (FTP),22 (SSH),23 (Telnet),389 (LDAP),443 (HTTPS),135 (EPMAP / DCOM),139 (NetBios),445 (Windows Shares),636 (LDAP over SSL),1433 (MSSQL) et 3389 (RDP).
	 .EXAMPLE
          PS C:\> Test-TCPPorts 10.34.145.12
		  PS C:\> Test-TCPPorts MonNomDeServeur
		  
		Ces commandes récupèrent la liste des ports ouverts et des ports fermés d'une machine distante.
		Les ports testés sont : 80 (HTTP),21 (FTP),22 (SSH),23 (Telnet),389 (LDAP),443 (HTTPS),135 (EPMAP / DCOM),139 (NetBios),445 (Windows Shares),636 (LDAP over SSL),1433 (MSSQL) et 3389 (RDP).
     .EXAMPLE
          PS C:\> "CATODC01", "CATOSV01", "CATOSV03" | Test-TCPPorts
		  
		Cette commande récupère la liste des ports ouverts et des ports fermés pour toutes les machines renvoyées par le pipeline.
	 .EXAMPLE
          PS C:\> Test-TCPPorts 10.34.145.12 -TCPportList 445, 80
		  
		Cette commande indique si les ports 445 et 80 sont ouverts.
	 .EXAMPLE
          PS C:\> Test-TCPPorts 10.34.145.12 -TCPportList 445, 80 -TimeOut 1000
		  
		Cette commande indique si les ports 445 et 80 sont ouverts. Le Timeout des requêtes TCP est fixé à 1 seconde (1000 ms).
	 .EXAMPLE
		  PS C:\> (1..254) | Test-TCPPorts -ComputerName {"192.168.1.$_"}
		  
		Cette commande récupère la liste des ports ouverts et des ports fermés pour chaque adresse IP comprise entre 192.168.1.1 et 192.168.1.254.
	 .OUTPUTS
	 	Selected.System.Management.Automation.PSCustomObject
     .LINK
		http://msdn.microsoft.com/en-us/library/system.net.sockets.tcpclient(v=vs.80).aspx
		http://msdn.microsoft.com/en-us/library/system.net.sockets.tcpclient_members(v=vs.80).aspx
		http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers
		www.via-powershell.fr
		www.powershell-scripting.com
	#>
	
    [CmdletBinding()]
    param(
		[Parameter(Position=0,Mandatory=$False,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
        [Alias("Name")]
        [string[]]$ComputerName = ".",
		[Parameter(Mandatory=$false)]
		[int[]]$TCPportList = @(80,21,22,23,389,443,135,139,445,636,1433,3389),
		[Parameter(Mandatory=$false)]
		[int]$TimeOut = 800
	)

	Begin{
		Function TestTCPPorts{
			param([String]$IPAddress, [String]$HostName)
			
			$ListOpenedPorts = @()
			$ListClosedPorts = @()
			
			Write-Debug "Traitement de chaque port de la liste de ports ..."
			foreach ($tcpport in $TCPportList) {
				$TCPclient = new-Object system.Net.Sockets.TcpClient
				$Connection = $TCPclient.BeginConnect($IPAddress,$tcpport,$null,$null)
				$Wait = $Connection.AsyncWaitHandle.WaitOne($TimeOut,$false)
				if(-not $Wait){
					Write-Debug "Le délai de connexion TCP fixé à '$TimeOut' ms est dépassé"
					Write-Debug "Fermeture de la connexion TCP ..."
					$TCPclient.Close()
					$ListClosedPorts += $tcpport
				}
				else{
					Write-Debug "Une réponse a été reçue dans le délai fixé à '$TimeOut' ms"
					Write-Debug "Fermeture de la connexion TCP ..."
					try{
						$TCPclient.EndConnect($Connection) | out-Null
						$ListOpenedPorts += $tcpport
					}
					catch{
						$ListClosedPorts += $tcpport
					}
					$TCPclient.Close()
				}
				
			}

			if(-not $ListOpenedPorts){
				$ListOpenedPorts = "Aucun port ouvert détecté"
			}
			if(-not $ListClosedPorts){
				$ListClosedPorts = "Aucun port fermé détecté"
			}
			
			Write-Debug "Génération d'un objet PowerShell personnalisé ..."
			$Answer = New-Object PSObject -Property @{
									DNSName = $HostName
									IPAddress = $IpAddress
									OpenedPorts = @($ListOpenedPorts)
									ClosedPorts = @($ListClosedPorts)
							}
			
			return $Answer	| Select-Object DNSName, IPAddress, OpenedPorts, ClosedPorts
		}
	}
	
	process{
		foreach($Computer in $ComputerName){
			$Resolve = Resolve-Computer $Computer
			if($Resolve){	
				Write-Debug "Test les ports '$TCPportList' sur '$Computer' ..."
				TestTCPPorts -IpAddress $Resolve.IpAddress -HostName $Resolve.HostName
			}
        }
	}
	
}

Function Get-NetBiosInfos{

	 <#
        .SYNOPSIS
            Récupère des informations NetBios à partir de la machine locale ou d'une machine distante.
        .DESCRIPTION
            Cette fonction permet de récupérer des informations NetBios à partir d'une machine distante.
            Ces informations NetBios sont le nom NetBios de la machine et le nom NetBios du domaine d'appartenance de la machine.
			Le nom DNS de la machine est également récupéré, s'il existe.
			Dans le cadre de la récupération des informations NetBios, c'est un Datagramme qui est trasmis sur le port UDP 137 de la machine.
			Seules les machines disposant de ces informations pourront répondre à ce type de requête (Windows, Linux + Samba, ...).
        .PARAMETER ComputerName
			Spécifie l'ordinateur sur lequel vous voulez exécuter l'opération de gestion. La valeur peut être un nom de domaine
			complet, un nom NetBIOS ou une adresse IP. Utilisez le nom de l'ordinateur local, localhost ou un point (.) pour 
			spécifier l'ordinateur local. L'ordinateur local est la valeur par défaut. Lorsque l'ordinateur distant se trouve 
			dans un domaine différent de l'utilisateur, vous devez utiliser un nom de domaine complet. Ce paramètre peut également 
			être redirigé vers l'applet de commande.

			Ce paramètre ne s'appuie pas sur la communication à distance Windows PowerShell, laquelle utilise le service Gestion
			des services Web). Vous pouvez utiliser le paramètre ComputerName de Get-NetBiosInfos même si votre ordinateur n'est
			pas configuré pour exécuter des commandes distantes du service Gestion des services Web.
		.EXAMPLE
			PS C:\> Get-NetBiosInfos
			PS C:\> Get-NetBiosInfos .
			PS C:\> Get-NetBiosInfos localhost

            Renvoie les informations NetBios (Nom et Domaine) de la machine locale.
        .EXAMPLE
            PS C:\> Get-NetBiosInfos 192.168.1.2

            Renvoie les informations NetBios (Nom et Domaine) de la machine correspondant à l'adresse IP 192.168.1.2 :
                - Si celle-ci est joignable sur le réseau ;
                - Si la machine dispose de ces informations.
        .EXAMPLE
            PS C:\> "192.168.1.2", "192.168.1.3" | Get-NetBiosInfos

            Renvoie les informations NetBios (Nom et Domaine) de la machine correspondant aux adresses renvoyées par le pipeline.
        .EXAMPLE
            'toto', '127.0.0.1', '192.168.0.1', '192.168.1.2', '192.168.1.1', '192.168.1.31' | Get-NetBiosInfos -ErrorAction SilentlyContinue

			Renvoie les informations NetBios (Nom et Domaine) de la machine correspondant aux adresses renvoyées par le pipeline.
            Les erreurs ne sont pas tracées.
		.EXAMPLE
          	PS C:\> (1..254) | Get-NetBiosInfos -ComputerName {"192.168.1.$_"}
		  
			Cette commande renvoie les informations NetBios (Nom et Domaine) pour chaque adresse IP comprise entre 192.168.1.1 et 192.168.1.254.
        .OUTPUTS
            Selected.System.Management.Automation.PSCustomObject
            Peut être une valeur $null : Cas de machine(s) qui ne répond(ent) pas ou qui ne dispos(ent) pas d'informations NetBios.
	    .LINK
			http://msdn.microsoft.com/en-us/library/system.net.sockets(v=vs.80).aspx
			http://en.wikipedia.org/wiki/List_of_TCP_and_UDP_port_numbers
			www.via-powershell.fr
			www.powershell-scripting.com
    #>
    [CmdletBinding()]
    param(
		[Parameter(Position=0,Mandatory=$False,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
        [Alias("Name")]
        [string[]]$ComputerName = "."
    )

    begin{
        Function GetNBTInfos{
		
			param([String]$IpAddress,[String]$HostName)
			
            Write-Debug "Définition du port UDP 137, port de destination pour la requête NetBios"
            [int]$Port = 137

            Write-Debug "Création de l'objet qui servira à stocker les informations NetBios"
            $TargetInfos = New-Object PSObject -Property @{DNSName = $HostName;IPAddress = $IpAddress ;NetBiosName = "";NetBiosDomain = ""}
            $Target = New-Object System.Net.IPEndPoint (([IPAddress]$IpAddress), $port)
            $SenderIPEndPoint = New-Object  Net.IPEndPoint([IPAddress]::Any, 0)
            $SenderEndPoint = [Net.EndPoint]$SenderIPEndPoint

            Write-Debug "Préparation à l'instance de la classe Socket ..."
            $Saddrf= [System.Net.Sockets.AddressFamily]::InterNetwork
            $Stype = [System.Net.Sockets.SocketType]::Dgram
            $Ptype = [System.Net.Sockets.ProtocolType]::UDP
            $Sock= New-Object System.Net.Sockets.Socket $saddrf, $stype, $ptype
            Write-Debug "Définition des paramètres SendTimeOut et ReceiveTimeOut ..."
            $Sock.SendTimeOut = 1000
            $Sock.ReceiveTimeout = 1000

            Write-Debug "Création du buffer qui servira à 'lire' la réponse"
            $buffer = new-object byte[] 1KB
            Write-Debug "Définition du datagram pour une requête NetBios"
            [Byte[]] $namerequest =  0x80, 0x94, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00,
                        0x00, 0x00, 0x00, 0x00, 0x20, 0x43, 0x4b, 0x41,
                        0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
                        0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
                        0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41,
                        0x41, 0x41, 0x41, 0x41, 0x41, 0x00, 0x00, 0x21,
                        0x00, 0x01 

            Write-Debug "Transmission du datagram vers '$ComputerName'"
            $a = $Sock.bind($SenderIPEndPoint)
            $a = $sock.sendto($namerequest, $Target) 
			
			$SockError = $false
			
            Try{
                $receiveByteCount = $Sock.ReceiveFrom($buffer, [Ref]$SenderEndPoint)
            }
            Catch [System.ArgumentNullException]{
                Write-Debug "La lecture du Buffer n'a pas fonctionné : $($_.Exception.Message)"
                Write-Error "'$ComputerName' : Au moins un des arguments ('buffer' ou 'remoteEP') vaut '`$null'."
                $SockError = $true
            }
            Catch [System.Net.Sockets.SocketException]{
                Write-Debug "La lecture du Buffer n'a pas fonctionné : $($_.Exception.Message)"
                Write-Error "'$ComputerName' : Une erreur s'est produite lors de la tentative d'accès au socket."
                $SockError = $true
            }
            Catch [System.ObjectDisposedException]{
                Write-Debug "La lecture du Buffer n'a pas fonctionné : $($_.Exception.Message)"
                Write-Error "'$ComputerName' : Le socket a été fermé."
                $SockError = $true
            }
            Catch [System.Security.SecurityException]{
                Write-Debug "La lecture du Buffer n'a pas fonctionné : $_.Exception.Message"
                Write-Error "'$ComputerName' : Un appelant de la pile des appels ne dispose pas des autorisations requises."
                $SockError = $true
            }
            Catch {
                Write-Debug "La lecture du Buffer n'a pas fonctionné : $_.Exception.Message"
                Write-Error "'$ComputerName' : Une erreur s'est produite ($_.Exception.Message)"
                $SockError = $true
            }

            if($receiveByteCount -ge 90 -and -not $SockError){
                $Encoding = New-Object System.Text.ASCIIEncoding
                Write-Debug "Lecture du nom NetBios de la machine, dans le Buffer ..."
                $TargetInfos.NetBiosName = ($Encoding.GetString($buffer, 57, 15)).Trim()
                Write-Debug "Lecture du nom NetBios du domaine de la machine, dans le Buffer ..."
                $TargetInfos.NetBiosDomain = ($Encoding.GetString($buffer, 75, 15)).Trim()
            }
			else{
				Write-Debug "La machine '$ComputerName' ne répond pas à la requête NetBios ..."
			}
			
            Write-Debug "Retourne l'objet comprenant les propriétés IPAddress, NetBiosName et NetBiosDomain ..."
            return $TargetInfos | Select-Object DNSName, IPAddress, NetBiosName, NetBiosDomain
        }

    }
	
    process{
		foreach($Computer in $ComputerName){
			$Resolve = Resolve-Computer $Computer
			if($Resolve){	
				Write-Debug "Récupération des informations NetBios sur '$Computer' ..."
				GetNBTInfos -IpAddress $Resolve.IpAddress -HostName $Resolve.HostName
			}
        }
	}
}



Function Get-IPInfos {
	<#
     .SYNOPSIS
		Permet de récupérer des informations NetBios, DNS et ports ouverts / fermés, à partir d'une adresse IP ou d'un nom d'équipement.
     .DESCRIPTION
        La cmdlet Get-IPInfos récupère des informations relatives à la résolution de nom via NetBios et via DNS.
		Elle permet également de déterminer les ports ouverts et fermés, selon une liste prédéfinie par défaut :
		80 (HTTP),21 (FTP),22 (SSH),23 (Telnet),389 (LDAP),443 (HTTPS),135 (EPMAP / DCOM),139 (NetBios),445 (Windows Shares),636 (LDAP over SSL),1433 (MSSQL) et 3389 (RDP).
		
		S'agissant d'un module, il est possible d'appeler directement les sous fonctions appelées par la Cmdlet Get-IPInfos, en utilisant leurs cmdlets respectives : 
		Test-TCPPorts et Get-NetBiosInfos.
     .PARAMETER  ComputerName
		Spécifie l'ordinateur sur lequel vous voulez exécuter l'opération de gestion. La valeur peut être un nom de domaine
		complet, un nom NetBIOS ou une adresse IP. Utilisez le nom de l'ordinateur local, localhost ou un point (.) pour 
		spécifier l'ordinateur local. L'ordinateur local est la valeur par défaut. Lorsque l'ordinateur distant se trouve 
		dans un domaine différent de l'utilisateur, vous devez utiliser un nom de domaine complet. Ce paramètre peut également 
		être redirigé vers l'applet de commande.

		Ce paramètre ne s'appuie pas sur la communication à distance Windows PowerShell, laquelle utilise le service Gestion
		des services Web). Vous pouvez utiliser le paramètre ComputerName de Get-IPInfos même si votre ordinateur n'est
		pas configuré pour exécuter des commandes distantes du service Gestion des services Web.
	 .EXAMPLE
          PS C:\> Get-IPInfos 
		  PS C:\> Get-IPInfos .
		  PS C:\> Get-IPInfos localhost
		  
		Ces commandes récupèrent les informations de la machine locale : DNS, NetBios et ports ouverts / fermés.
     .EXAMPLE
          PS C:\> Get-IPInfos 10.34.145.12
		  PS C:\> Get-IPInfos MonNomDeServeur
		  
		Ces commandes récupèrent les informations d'une machine distante : DNS, NetBios et ports ouverts / fermés.
     .EXAMPLE
          PS C:\> Get-IPInfos CATODC01, CATOSV02, 192.168.1.12
		  
		Cette commande récupère des informations DNS, NetBios et ports ouverts / fermés pour les 3 machines spécifiées.
	.EXAMPLE
          PS C:\> "CATODC01", "CATOSV02" | Get-IPInfos
		  
		Cette commande récupère des informations DNS, NetBios et ports ouverts / fermés pour les 2 noms de machines renvoyés par la pipeline.
	.EXAMPLE
          PS C:\> Get-Content .\Servers.txt | Get-IPInfos
		  
		Cette commande récupère des informations DNS, NetBios et ports ouverts / fermés pour tous les noms de machines spécifiés dans le fichier texte.		
	.EXAMPLE
          PS C:\> (1..254) | Get-IPInfos -ComputerName {"192.168.1.$_"}
		  
		Cette commande récupère des informations DNS, NetBios et ports ouverts / fermés pour toutes les adresses IP comprises entre 192.168.1.1 et 192.168.1.254.
	.LINK
          www.via-powershell.fr
		  www.powershell-scripting.com
	#>
	
    [CmdletBinding()]
    param(
		[Parameter(Position=0,Mandatory=$False,ValueFromPipeline=$True,ValueFromPipelineByPropertyName=$True)]
        [Alias("Name")]
        [string[]]$ComputerName = "."
    )
	
	Begin{
		Function GetIPInfos{
			param([String]$ComputerName)
			$Myports = (Test-TCPPorts -ComputerName $ComputerName)
			if($Myports){
				$MyNetBiosInfos = (Get-NetBiosInfos -ComputerName $ComputerName)
				$Answer = New-Object PSObject -Property @{
													IPAddress = $Myports.IPAddress
													NetBiosName = $MyNetBiosInfos.NetBiosName
													DNSName = $Myports.DNSName
													NetBiosDomain = $MyNetBiosInfos.NetBiosDomain
													OpenedPorts = $Myports.OpenedPorts
													ClosedPorts = $Myports.ClosedPorts
												}
												
				return $Answer | Select-Object IPAddress, DNSName, NetBiosName, NetBiosDomain, OpenedPorts, ClosedPorts
			}
		}
	}
	
	process{
		foreach($Computer in $ComputerName){
			GetIPInfos -ComputerName $Computer
        }
	}
}

# Ces alias sont fournis à titre indicatif et peuvent être modifiés à loisirs ...
set-alias -name gipconfig -value Get-IPConfig -Description "IPInfos alias"
set-alias -name gipconf -value Get-IPConfig -Description "IPInfos alias"
set-alias -name gipc -value Get-IPConfig -Description "IPInfos alias"
set-alias -name tTCP -value Test-TCPPorts -Description "IPInfos alias"
set-alias -name gNB -value Get-NetBiosInfos -Description "IPInfos alias"
set-alias -name gIP -value Get-IPInfos -Description "IPInfos alias"

Export-ModuleMember -Alias * -Function Get-IPConfig, Test-TCPPorts, Get-NetBiosInfos, Get-IPInfos

<#
  .Synopsis
   Ce module fournie des Cmdlets qui permettent de récupérer des informations IP à partir de la machine locale ou de machines distantes.
  .Description
   Ce module fournie 4 cmdlets : Test-TCPPorts, Get-NetBiosInfos, Get-IPInfos (Cette dernière appelle les Cmdlets Test-TCPPorts et Get-NetBiosInfos) et Get-IPConfig.
  .Notes
   NAME: IPInfos.psm1
   AUTHOR: Matthew BETTON
   LASTEDIT: 05/08/2012
   KEYWORDS: IPconfig, Netbios, DNS, Ports, sockets
  .Link
   http://www.via-powershell.fr
   http://www.powershell-scripting.com
 #Requires -Version 2.0
#> 


